@@ -1,17 +1,17 @@
-#!/usr/bin/env python3
+#!/usr/bin/python3
 
 import sys
-sys.path.append("debian/lib/python")
-
+import deb822
+import glob
 import os
 import os.path
 import re
 import shutil
 import subprocess
 import time
+import warnings
 
 from debian_linux.debian import Changelog, VersionLinux
-from debian_linux.patches import PatchSeries
 
 
 class Main(object):
@@ -31,7 +31,8 @@ class Main(object):
         if self.version_dfsg is None:
             self.version_dfsg = '0'
 
-        self.log('Using source name %s, version %s, dfsg %s\n' % (source, version.upstream, self.version_dfsg))
+        self.log('Using source name %s, version %s, dfsg %s\n' %
+                 (source, version.upstream, self.version_dfsg))
 
         self.orig = '%s-%s' % (source, version.upstream)
         self.orig_tar = '%s_%s.orig.tar.xz' % (source, version.upstream)
@@ -49,7 +50,7 @@ class Main(object):
             if len(self.input_files) > 1:
                 self.upstream_patch(self.input_files[1])
 
-            # debian_patch() will change file mtimes.  Capture the
+            # exclude_files() will change dir mtimes.  Capture the
             # original release time so we can apply it to the final
             # tarball.  Note this doesn't work in case we apply an
             # upstream patch, as that doesn't carry a release time.
@@ -59,7 +60,7 @@ class Main(object):
                     os.stat(os.path.join(self.dir, self.orig, 'Makefile'))
                     .st_mtime))
 
-            self.debian_patch()
+            self.exclude_files()
             os.umask(old_umask)
             self.tar(orig_date)
         finally:
@@ -74,7 +75,7 @@ class Main(object):
         verify_proc = subprocess.Popen(['git',
                                         '-c', 'gpg.program=%s' % gpg_wrapper,
                                         'tag', '-v', self.tag],
-                                        cwd=input_repo)
+                                       cwd=input_repo)
         if verify_proc.wait():
             raise RuntimeError("GPG tag verification failed")
 
@@ -92,7 +93,9 @@ class Main(object):
 
     def upstream_extract(self, input_tar):
         self.log("Extracting tarball %s\n" % input_tar)
-        match = re.match(r'(^|.*/)(?P<dir>linux-\d+\.\d+(\.\d+)?(-\S+)?)\.tar(\.(?P<extension>(bz2|gz|xz)))?$', input_tar)
+        match = re.match(r'(^|.*/)(?P<dir>linux-\d+\.\d+(\.\d+)?(-\S+)?)\.tar'
+                         r'(\.(?P<extension>(bz2|gz|xz)))?$',
+                         input_tar)
         if not match:
             raise RuntimeError("Can't identify name of tarball")
 
@@ -101,11 +104,14 @@ class Main(object):
         if subprocess.Popen(cmdline).wait():
             raise RuntimeError("Can't extract tarball")
 
-        os.rename(os.path.join(self.dir, match.group('dir')), os.path.join(self.dir, self.orig))
+        os.rename(os.path.join(self.dir, match.group('dir')),
+                  os.path.join(self.dir, self.orig))
 
     def upstream_patch(self, input_patch):
         self.log("Patching source with %s\n" % input_patch)
-        match = re.match(r'(^|.*/)patch-\d+\.\d+(\.\d+)?(-\S+?)?(\.(?P<extension>(bz2|gz|xz)))?$', input_patch)
+        match = re.match(r'(^|.*/)patch-\d+\.\d+(\.\d+)?(-\S+?)?'
+                         r'(\.(?P<extension>(bz2|gz|xz)))?$',
+                         input_patch)
         if not match:
             raise RuntimeError("Can't identify name of patch")
         cmdline = []
@@ -118,16 +124,28 @@ class Main(object):
         else:
             cmdline.append('cat')
         cmdline.append(input_patch)
-        cmdline.append('| (cd %s; patch -p1 -f -s -t --no-backup-if-mismatch)' % os.path.join(self.dir, self.orig))
+        cmdline.append('| (cd %s; patch -p1 -f -s -t --no-backup-if-mismatch)'
+                       % os.path.join(self.dir, self.orig))
         if os.spawnv(os.P_WAIT, '/bin/sh', ['sh', '-c', ' '.join(cmdline)]):
             raise RuntimeError("Can't patch source")
 
-    def debian_patch(self):
-        name = "orig"
-        self.log("Patching source with debian patch (series %s)\n" % name)
-        fp = open("debian/patches/series-" + name)
-        series = PatchSeries(name, "debian/patches", fp)
-        series(dir=os.path.join(self.dir, self.orig))
+    def exclude_files(self):
+        self.log("Excluding file patterns specified in debian/copyright\n")
+        with open("debian/copyright") as f:
+            header = deb822.Deb822(f)
+        patterns = header.get("Files-Excluded", '').strip().split()
+        for pattern in patterns:
+            matched = False
+            for name in glob.glob(os.path.join(self.dir, self.orig, pattern)):
+                try:
+                    shutil.rmtree(name)
+                except NotADirectoryError:
+                    os.unlink(name)
+                matched = True
+            if not matched:
+                warnings.warn("Exclusion pattern '%s' did not match anything"
+                              % pattern,
+                              RuntimeWarning)
 
     def tar(self, orig_date):
         out = os.path.join("../orig", self.orig_tar)
@@ -141,29 +159,44 @@ class Main(object):
         except OSError:
             pass
         self.log("Generate tarball %s\n" % out)
-        cmdline = '''(cd '%s' && find '%s' -print0) |
-                     LC_ALL=C sort -z |
-                     tar -C '%s' --no-recursion --null -T - --mtime '%s' --owner root --group root -caf '%s'
-                  ''' % (self.dir, self.orig, self.dir, orig_date, out)
+
+        env = os.environ.copy()
+        env.update({
+            'LC_ALL': 'C',
+        })
+        cmd = [
+            'tar',
+            '-C', self.dir,
+            '--sort=name',
+            '--mtime={}'.format(orig_date),
+            '--owner=root',
+            '--group=root',
+            '--use-compress-program=xz -T0',
+            '-cf',
+            out, self.orig,
+        ]
+
         try:
-            if os.spawnv(os.P_WAIT, '/bin/sh', ['sh', '-c', cmdline]):
-                raise RuntimeError("Can't patch source")
+            subprocess.run(cmd, env=env, check=True)
             os.chmod(out, 0o644)
-        except:
+        except BaseException:
             try:
                 os.unlink(out)
             except OSError:
                 pass
             raise
         try:
-            os.symlink(os.path.join('orig', self.orig_tar), os.path.join('..', self.orig_tar))
+            os.symlink(os.path.join('orig', self.orig_tar),
+                       os.path.join('..', self.orig_tar))
         except OSError:
             pass
 
+
 if __name__ == '__main__':
     from optparse import OptionParser
     parser = OptionParser(usage="%prog [OPTION]... {TAR [PATCH] | REPO}")
-    parser.add_option("-V", "--override-version", dest="override_version", help="Override version", metavar="VERSION")
+    parser.add_option("-V", "--override-version", dest="override_version",
+                      help="Override version", metavar="VERSION")
     options, args = parser.parse_args()
 
     assert 1 <= len(args) <= 2
