@@ -1,7 +1,9 @@
 import codecs
+import re
 from collections import OrderedDict
 
-from .debian import *
+from .debian import Changelog, PackageArchitecture, PackageDescription, \
+    PackageRelation, Version
 
 
 class PackagesList(OrderedDict):
@@ -63,10 +65,6 @@ class Makefile(object):
 
 
 class MakeFlags(dict):
-    def __repr__(self):
-        repr = super(flags, self).__repr__()
-        return "%s(%s)" % (self.__class__.__name__, repr)
-
     def __str__(self):
         return ' '.join("%s='%s'" % i for i in sorted(self.items()))
 
@@ -91,15 +89,16 @@ class Gencontrol(object):
         self.do_main(packages, makefile)
         self.do_extra(packages, makefile)
 
+        self.merge_build_depends(packages)
         self.write(packages, makefile)
 
     def do_source(self, packages):
         source = self.templates["control.source"][0]
-        source['Source'] = self.changelog[0].source
+        if not source.get('Source'):
+            source['Source'] = self.changelog[0].source
         packages['source'] = self.process_package(source, self.vars)
 
     def do_main(self, packages, makefile):
-        config_entry = self.config['base', ]
         vars = self.vars.copy()
 
         makeflags = MakeFlags()
@@ -114,19 +113,25 @@ class Gencontrol(object):
         pass
 
     def do_main_makefile(self, makefile, makeflags, extra):
-        makefile.add('build-indep', cmds=["$(MAKE) -f debian/rules.real build-indep %s" % makeflags])
-        makefile.add('binary-indep', cmds=["$(MAKE) -f debian/rules.real binary-indep %s" % makeflags])
+        makefile.add('build-indep',
+                     cmds=["$(MAKE) -f debian/rules.real build-indep %s" %
+                           makeflags])
+        makefile.add('binary-indep',
+                     cmds=["$(MAKE) -f debian/rules.real binary-indep %s" %
+                           makeflags])
 
     def do_main_packages(self, packages, vars, makeflags, extra):
         pass
 
     def do_main_recurse(self, packages, makefile, vars, makeflags, extra):
         for featureset in self.config['base', ]['featuresets']:
-            if self.config.merge('base', None, featureset).get('enabled', True):
+            if self.config.merge('base', None, featureset) \
+                          .get('enabled', True):
                 self.do_indep_featureset(packages, makefile, featureset,
                                          vars.copy(), makeflags.copy(), extra)
         for arch in iter(self.config['base', ]['arches']):
-            self.do_arch(packages, makefile, arch, vars.copy(), makeflags.copy(), extra)
+            self.do_arch(packages, makefile, arch, vars.copy(),
+                         makeflags.copy(), extra)
 
     def do_extra(self, packages, makefile):
         templates_extra = self.templates.get("control.extra", None)
@@ -145,12 +150,15 @@ class Gencontrol(object):
         for arch in sorted(extra_arches.keys()):
             cmds = []
             for i in extra_arches[arch]:
-                cmds.append("$(MAKE) -f debian/rules.real install-dummy ARCH='%s' DH_OPTIONS='-p%s'" % (arch, i['Package']))
-            makefile.add('binary-arch_%s' % arch, ['binary-arch_%s_extra' % arch])
-            makefile.add("binary-arch_%s_extra" % arch, cmds = cmds)
+                cmds.append("$(MAKE) -f debian/rules.real install-dummy "
+                            "ARCH='%s' DH_OPTIONS='-p%s'" %
+                            (arch, i['Package']))
+            makefile.add('binary-arch_%s' % arch,
+                         ['binary-arch_%s_extra' % arch])
+            makefile.add("binary-arch_%s_extra" % arch, cmds=cmds)
 
     def do_indep_featureset(self, packages, makefile, featureset, vars,
-                             makeflags, extra):
+                            makeflags, extra):
         vars['localversion'] = ''
         if featureset != 'none':
             vars['localversion'] = '-' + featureset
@@ -200,14 +208,18 @@ class Gencontrol(object):
             makefile.add(target1, [target2])
             makefile.add(target2, [target3])
 
-    def do_arch_packages(self, packages, makefile, arch, vars, makeflags, extra):
+    def do_arch_packages(self, packages, makefile, arch, vars, makeflags,
+                         extra):
         pass
 
-    def do_arch_recurse(self, packages, makefile, arch, vars, makeflags, extra):
+    def do_arch_recurse(self, packages, makefile, arch, vars, makeflags,
+                        extra):
         for featureset in self.config['base', arch].get('featuresets', ()):
-            self.do_featureset(packages, makefile, arch, featureset, vars.copy(), makeflags.copy(), extra)
+            self.do_featureset(packages, makefile, arch, featureset,
+                               vars.copy(), makeflags.copy(), extra)
 
-    def do_featureset(self, packages, makefile, arch, featureset, vars, makeflags, extra):
+    def do_featureset(self, packages, makefile, arch, featureset, vars,
+                      makeflags, extra):
         config_base = self.config.merge('base', arch, featureset)
         if not config_base.get('enabled', True):
             return
@@ -217,14 +229,18 @@ class Gencontrol(object):
             vars['localversion'] = '-' + featureset
 
         self.do_featureset_setup(vars, makeflags, arch, featureset, extra)
-        self.do_featureset_makefile(makefile, arch, featureset, makeflags, extra)
-        self.do_featureset_packages(packages, makefile, arch, featureset, vars, makeflags, extra)
-        self.do_featureset_recurse(packages, makefile, arch, featureset, vars, makeflags, extra)
+        self.do_featureset_makefile(makefile, arch, featureset, makeflags,
+                                    extra)
+        self.do_featureset_packages(packages, makefile, arch, featureset, vars,
+                                    makeflags, extra)
+        self.do_featureset_recurse(packages, makefile, arch, featureset, vars,
+                                   makeflags, extra)
 
     def do_featureset_setup(self, vars, makeflags, arch, featureset, extra):
         pass
 
-    def do_featureset_makefile(self, makefile, arch, featureset, makeflags, extra):
+    def do_featureset_makefile(self, makefile, arch, featureset, makeflags,
+                               extra):
         makeflags['FEATURESET'] = featureset
 
         for i in self.makefile_targets:
@@ -234,31 +250,38 @@ class Gencontrol(object):
             makefile.add(target1, [target2])
             makefile.add(target2, [target3])
 
-    def do_featureset_packages(self, packages, makefile, arch, featureset, vars, makeflags, extra):
+    def do_featureset_packages(self, packages, makefile, arch, featureset,
+                               vars, makeflags, extra):
         pass
 
-    def do_featureset_recurse(self, packages, makefile, arch, featureset, vars, makeflags, extra):
+    def do_featureset_recurse(self, packages, makefile, arch, featureset, vars,
+                              makeflags, extra):
         for flavour in self.config['base', arch, featureset]['flavours']:
-            self.do_flavour(packages, makefile, arch, featureset, flavour, vars.copy(), makeflags.copy(), extra)
-
-    def do_flavour(self, packages, makefile, arch, featureset, flavour, vars, makeflags, extra):
-        config_base = self.config.merge('base', arch, featureset, flavour)
+            self.do_flavour(packages, makefile, arch, featureset, flavour,
+                            vars.copy(), makeflags.copy(), extra)
 
+    def do_flavour(self, packages, makefile, arch, featureset, flavour, vars,
+                   makeflags, extra):
         vars['localversion'] += '-' + flavour
 
-        self.do_flavour_setup(vars, makeflags, arch, featureset, flavour, extra)
-        self.do_flavour_makefile(makefile, arch, featureset, flavour, makeflags, extra)
-        self.do_flavour_packages(packages, makefile, arch, featureset, flavour, vars, makeflags, extra)
+        self.do_flavour_setup(vars, makeflags, arch, featureset, flavour,
+                              extra)
+        self.do_flavour_makefile(makefile, arch, featureset, flavour,
+                                 makeflags, extra)
+        self.do_flavour_packages(packages, makefile, arch, featureset, flavour,
+                                 vars, makeflags, extra)
 
-    def do_flavour_setup(self, vars, makeflags, arch, featureset, flavour, extra):
+    def do_flavour_setup(self, vars, makeflags, arch, featureset, flavour,
+                         extra):
         for i in (
             ('kernel-arch', 'KERNEL_ARCH'),
             ('localversion', 'LOCALVERSION'),
-        ):  
+        ):
             if i[0] in vars:
                 makeflags[i[1]] = vars[i[0]]
 
-    def do_flavour_makefile(self, makefile, arch, featureset, flavour, makeflags, extra):
+    def do_flavour_makefile(self, makefile, arch, featureset, flavour,
+                            makeflags, extra):
         makeflags['FLAVOUR'] = flavour
 
         for i in self.makefile_targets:
@@ -268,7 +291,8 @@ class Gencontrol(object):
             makefile.add(target1, [target2])
             makefile.add(target2, [target3])
 
-    def do_flavour_packages(self, packages, makefile, arch, featureset, flavour, vars, makeflags, extra):
+    def do_flavour_packages(self, packages, makefile, arch, featureset,
+                            flavour, vars, makeflags, extra):
         pass
 
     def process_relation(self, dep, vars):
@@ -312,20 +336,43 @@ class Gencontrol(object):
 
         return re.sub(r'@([-_a-z0-9]+)@', subst, str(s))
 
+    def merge_build_depends(self, packages):
+        # Merge Build-Depends pseudo-fields from binary packages into the
+        # source package
+        source = packages["source"]
+        arch_all = PackageArchitecture("all")
+        for name, package in packages.items():
+            if name == "source":
+                continue
+            dep = package.get("Build-Depends")
+            if not dep:
+                continue
+            del package["Build-Depends"]
+            for group in dep:
+                for item in group:
+                    if package["Architecture"] != arch_all and not item.arches:
+                        item.arches = sorted(package["Architecture"])
+                    if package.get("Build-Profiles") and not item.restrictions:
+                        profiles = package["Build-Profiles"]
+                        assert profiles[0] == "<" and profiles[-1] == ">"
+                        item.restrictions = re.split(r"\s+", profiles[1:-1])
+            if package["Architecture"] == arch_all:
+                dep_type = "Build-Depends-Indep"
+            else:
+                dep_type = "Build-Depends-Arch"
+            if dep_type not in source:
+                source[dep_type] = PackageRelation()
+            source[dep_type].extend(dep)
+
     def write(self, packages, makefile):
         self.write_control(packages.values())
         self.write_makefile(makefile)
 
-    def write_config(self):
-        f = file("debian/config.dump", 'w')
-        self.config.write(f)
-        f.close()
+    def write_control(self, list, name='debian/control'):
+        self.write_rfc822(codecs.open(name, 'w', 'utf-8'), list)
 
-    def write_control(self, list):
-        self.write_rfc822(codecs.open("debian/control", 'w', 'utf-8'), list)
-
-    def write_makefile(self, makefile):
-        f = open("debian/rules.gen", 'w')
+    def write_makefile(self, makefile, name='debian/rules.gen'):
+        f = open(name, 'w')
         makefile.write(f)
         f.close()
 
@@ -335,6 +382,7 @@ class Gencontrol(object):
                 f.write(u"%s: %s\n" % (key, value))
             f.write('\n')
 
+
 def merge_packages(packages, new, arch):
     for new_package in new:
         name = new_package['Package']
@@ -342,7 +390,8 @@ def merge_packages(packages, new, arch):
             package = packages.get(name)
             package['Architecture'].add(arch)
 
-            for field in 'Depends', 'Provides', 'Suggests', 'Recommends', 'Conflicts':
+            for field in ('Depends', 'Provides', 'Suggests', 'Recommends',
+                          'Conflicts'):
                 if field in new_package:
                     if field in package:
                         v = package[field]
